"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _keys = require("babel-runtime/core-js/object/keys");

var _keys2 = _interopRequireDefault(_keys);

var _defineProperty2 = require("babel-runtime/helpers/defineProperty");

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _values = require("babel-runtime/core-js/object/values");

var _values2 = _interopRequireDefault(_values);

var _extends3 = require("babel-runtime/helpers/extends");

var _extends4 = _interopRequireDefault(_extends3);

var _objectWithoutProperties2 = require("babel-runtime/helpers/objectWithoutProperties");

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

var _toConsumableArray2 = require("babel-runtime/helpers/toConsumableArray");

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

exports.default = function (_ref) {
  var t = _ref.types;

  var buildScope = function buildScope(path) {
    var scope = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

    var functionParent = path.getFunctionParent();
    if (!functionParent) {
      return scope;
    }
    if (functionParent.isFunctionDeclaration()) {
      scope.push(functionParent.node.id.name);
    } else if (t.isClassProperty(functionParent) || t.isClassMethod(functionParent)) {
      scope.push(functionParent.node.key.name);
      var classBody = functionParent.findParent(function (path) {
        return path.isClassDeclaration() || path.isClassExpression();
      });
      if (classBody) {
        scope.push(classBody.node.id.name);
        return buildScope(classBody, scope);
      }
    } else if (functionParent.isArrowFunctionExpression()) {
      var arrFuncParent = functionParent.findParent(function (path) {
        return path;
      });
      if (t.isVariableDeclarator(arrFuncParent)) {
        scope.push(arrFuncParent.node.id.name);
      } else if (t.isClassProperty(arrFuncParent) || t.isClassMethod(arrFuncParent)) {
        scope.push(arrFuncParent.node.key.name);
        var _classBody = arrFuncParent.findParent(function (path) {
          return path.isClassDeclaration() || path.isClassExpression();
        });
        if (_classBody) {
          scope.push(_classBody.node.id.name);
          return buildScope(_classBody, scope);
        }
      } else if (t.isCallExpression(arrFuncParent)) {
        var callee = arrFuncParent.node.callee;

        scope.push("[" + callee.object.name + "." + callee.property.name + "]");
      }
    }
    return buildScope(functionParent, scope);
  };

  var injectVariableNames = function injectVariableNames() {
    var args = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

    return args.reduce(function (acc, arg) {
      if (t.isIdentifier(arg)) {
        return [].concat((0, _toConsumableArray3.default)(acc), [t.stringLiteral(arg.name), arg]);
      }
      return [].concat((0, _toConsumableArray3.default)(acc), [arg]);
    }, []);
  };

  var buildSettings = function buildSettings(opts) {
    var methods = opts.methods,
        flags = (0, _objectWithoutProperties3.default)(opts, ["methods"]);
    // output speards the flags over each method
    // in the future this could be expanded to allow method level config

    return (methods || defaultMethods).reduce(function (acc, curr) {
      return (0, _extends4.default)({}, acc, (0, _defineProperty3.default)({}, curr, (0, _values2.default)(flags).length ? flags : defaultSettings));
    }, {});
  };

  var isConsoleStatement = function isConsoleStatement(path) {
    return path.get("object").isIdentifier({ name: "console" });
  };

  var prependArguments = function prependArguments() {
    var args = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
    var value = arguments[1];

    args.unshift(t.stringLiteral(value));
  };

  return {
    name: "babel-plugin-captains-log", // not required
    visitor: {
      MemberExpression: function MemberExpression(path, _ref2) {
        var file = _ref2.file,
            opts = _ref2.opts;

        if (!isConsoleStatement(path)) {
          return;
        }
        var settings = buildSettings(opts || {});
        if (!(0, _keys2.default)(settings).includes(path.node.property.name)) {
          return;
        }
        var options = settings[path.node.property.name];
        // console.log(options);
        var parent = path.parent;
        if (t.isCallExpression(parent) && parent.arguments && Array.isArray(parent.arguments)) {
          // add variable names
          if (options.injectVariableName) {
            parent.arguments = injectVariableNames(parent.arguments);
          }
          // prepend filename
          if (options.injectFileName) {
            prependArguments(parent.arguments, file.opts.filename);
          }

          // prepend console statement scope
          if (options.injectScope) {
            var scope = buildScope(path);
            prependArguments(parent.arguments, scope.reverse().join(".") + ":");
          }
        }
      }
    }
  };
};

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var defaultSettings = {
  injectScope: true,
  injectVariableName: true,
  injectFileName: false
};

var defaultMethods = ["debug", "error", "exception", "info", "log", "warn"];